#include "I2Cdev.h"
/** Default constructor.

 */

 uint16_t I2Cdev::readTimeout = I2CDEV_DEFAULT_READ_TIMEOUT;
 bool I2Cdev::i2cModulisReady = false;


I2Cdev::I2Cdev() {

}

void I2Cdev::init(uint32_t speed, bool interrupt){
	#ifdef I2CDEV_SERIAL_DEBUG
       serial.print("I2C1::init()...");
   	#endif


	if(!i2cModulisReady){
	#ifdef I2CDEV_SERIAL_DEBUG
       serial.print(" !i2cModulisReady");
   	#endif
			i2cModulisReady = true;
			I2C_Module = I2C1;

			GPIO_InitTypeDef GPIO_InitStructure;
			I2C_InitTypeDef I2C_InitStructure;


			#ifdef I2CDEV_SERIAL_DEBUG
				serial.print(", enable tim2");
			#endif
			// fr timeoutfunktion wird der tim2 verwendet mit ISR-Aufrug alle 1ms
			tim2initNORMAL(8, 7999);
			tim2enableInterrupt();


			#ifdef I2CDEV_SERIAL_DEBUG
				serial.print(", I2C1 DeInit");
			#endif

			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);


			GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
			GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
			GPIO_Init(GPIOB, &GPIO_InitStructure);

			I2C_DeInit(I2C1);
			#ifdef I2CDEV_SERIAL_DEBUG
				serial.print(", enable GPIO AF");
			#endif


			#ifdef I2CDEV_SERIAL_DEBUG
				serial.print(", I2C1 Init");
			#endif
			I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
			I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
			I2C_InitStructure.I2C_ClockSpeed = speed;
			I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_16_9;
			I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
			I2C_InitStructure.I2C_OwnAddress1 = 0;
			I2C_Init(I2C1, &I2C_InitStructure);
			//I2C_StructInit(&I2C_InitStructure);
			if(interrupt == true){

				#ifdef I2CDEV_SERIAL_DEBUG
					serial.print(" enable I2C1-Interrupts");
				#endif
				NVIC_InitTypeDef NVIC_InitStructure;

				NVIC_InitStructure.NVIC_IRQChannel = I2C1_EV_IRQn;
				NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0; // 0== hchste Prioritt?
				NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
				NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
				NVIC_Init(&NVIC_InitStructure);


				NVIC_InitStructure.NVIC_IRQChannel = I2C1_ER_IRQn;
				NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
				NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
				NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
				NVIC_Init(&NVIC_InitStructure);
				I2C_ITConfig(I2C1, I2C_IT_EVT, ENABLE); //SB, ADDR, ADD10, STOPF, BTF, RxNE, TxE
				I2C_ITConfig(I2C1, I2C_IT_BUF, ENABLE);	//RxNE, TxE
														//..... unntig, wenn IT_EVT eh schon diese Flags einen Interrupt ausslst?
				I2C_ITConfig(I2C1, I2C_IT_ERR, ENABLE);
			}
			I2C_Cmd(I2C1, ENABLE);
		}
}

/** Read a single bit from an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param bitNum Bit position to read (0-7)
 * @param data Container for single bit value
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (true = success)
 */
int8_t I2Cdev::readBit(uint8_t devAddr, uint8_t regAddr, uint8_t bitNum, uint8_t *data, uint16_t timeout) {
    uint8_t b;
    uint8_t count = readByte(devAddr, regAddr, &b, timeout);
    *data = b & (1 << bitNum);
    return count;
}

/** Read a single bit from a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param bitNum Bit position to read (0-15)
 * @param data Container for single bit value
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (true = success)
 */
int8_t I2Cdev::readBitW(uint8_t devAddr, uint8_t regAddr, uint8_t bitNum, uint16_t *data, uint16_t timeout) {
    uint16_t b;
    uint8_t count = readWord(devAddr, regAddr, &b, timeout);
    *data = b & (1 << bitNum);
    return count;
}

/** Read multiple bits from an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param bitStart First bit position to read (0-7)
 * @param length Number of bits to read (not more than 8)
 * @param data Container for right-aligned value (i.e. '101' read from any bitStart position will equal 0x05)
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (true = success)
 */
int8_t I2Cdev::readBits(uint8_t devAddr, uint8_t regAddr, uint8_t bitStart, uint8_t length, uint8_t *data, uint16_t timeout) {
    // 01101001 read byte
    // 76543210 bit numbers
    //    xxx   args: bitStart=4, length=3
    //    010   masked
    //   -> 010 shifted
    uint8_t count, b;
    if ((count = readByte(devAddr, regAddr, &b, timeout)) != 0) {
        uint8_t mask = ((1 << length) - 1) << (bitStart - length + 1);
        b &= mask;
        b >>= (bitStart - length + 1);
        *data = b;
    }
    return count;
}

/** Read multiple bits from a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param bitStart First bit position to read (0-15)
 * @param length Number of bits to read (not more than 16)
 * @param data Container for right-aligned value (i.e. '101' read from any bitStart position will equal 0x05)
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (1 = success, 0 = failure, -1 = timeout)
 */
int8_t I2Cdev::readBitsW(uint8_t devAddr, uint8_t regAddr, uint8_t bitStart, uint8_t length, uint16_t *data, uint16_t timeout) {
    // 1101011001101001 read byte
    // fedcba9876543210 bit numbers
    //    xxx           args: bitStart=12, length=3
    //    010           masked
    //           -> 010 shifted
    uint8_t count;
    uint16_t w;
    if ((count = readWord(devAddr, regAddr, &w, timeout)) != 0) {
        uint16_t mask = ((1 << length) - 1) << (bitStart - length + 1);
        w &= mask;
        w >>= (bitStart - length + 1);
        *data = w;
    }
    return count;
}

/** Read single byte from an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param data Container for byte value read from device
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (true = success)
 */
int8_t I2Cdev::readByte(uint8_t devAddr, uint8_t regAddr, uint8_t *data, uint16_t timeout) {
    return readBytes(devAddr, regAddr, 1, data, timeout);
}

/** Read single word from a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to read from
 * @param data Container for word value read from device
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Status of read operation (true = success)
 */
int8_t I2Cdev::readWord(uint8_t devAddr, uint8_t regAddr, uint16_t *data, uint16_t timeout) {
    return readWords(devAddr, regAddr, 1, data, timeout);
}

/** Read multiple bytes from an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr First register regAddr to read from
 * @param length Number of bytes to read
 * @param data Buffer to store read data in
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Number of bytes read (-1 indicates failure)
 */

int8_t I2Cdev::readBytes(uint8_t devAddr, uint8_t myregAddr, uint8_t length, uint8_t *data, uint16_t Ftimeout) {
    #ifdef I2CDEV_SERIAL_DEBUG
        serial.print("I2Cdev (0x");
        serial.print(devAddr, 16);
        serial.print(") reading ");
        serial.print(length, 10);
        serial.print(" bytes from 0x");
        serial.print(myregAddr, 16);
        serial.print("...");
    #endif

    int8_t count = 0;

    uint16_t tStart = millis();

   	while(i2cBusyFlag){
   		if((uint16_t)(millis() - tStart) >= Ftimeout){
   			//Timeout
   			serial.println("  !!!TO, readBytes, i2cBusyFlag1");
   			return -1;
   		}
   	}

	for (uint8_t k = 0; k < length; k += min(length, BUFFER_LENGTH)) {

		while(i2cBusyFlag){
			//serial.println("waiting1");
			if((uint16_t)(millis() - tStart) >= Ftimeout){
				//Timeout
				serial.println("  !!!TO, readBytes, i2cBusyFlag");
				return -1;
			}
		}
		while(I2C_GetFlagStatus(I2C_Module,I2C_FLAG_BUSY)== SET){
			if((uint16_t)(millis() - tStart) >= Ftimeout){
				//Timeout
				serial.println("  !!!TO, readBytes, Bus busy");
				return -1;
			}
		}

		deviceAddress = devAddr;
		i2cDirectionWrite = 1;
		i2cBusyFlag = 1;


		regAddr = myregAddr + k;

		//debugflag = 1;

		rxBufferIndex = 0;
		rxRemainCount = min(length, BUFFER_LENGTH);

		txCount = 0;
		txBufferIndex = 0;

			#ifdef I2CDEV_SERIAL_DEBUG
				serial.endl();
				serial.printlnVal("deviceAddress", deviceAddress);
				serial.printlnVal("i2cDirectionWrite", i2cDirectionWrite);
				serial.printlnVal("i2cBusyFlag", i2cBusyFlag);
				serial.printlnVal("regAddr", regAddr);
				serial.printlnVal("rxBufferIndex", rxBufferIndex);
				serial.printlnVal("rxRemainCount", rxRemainCount);
				serial.printlnVal("txCount", txCount);
				serial.printlnVal("txBufferIndex", txBufferIndex);

			#endif

		I2C_AcknowledgeConfig(I2C_Module, ENABLE);
		I2C_GenerateSTART(I2C_Module, ENABLE);
		while(i2cBusyFlag){
			//serial.println("waiting1");
			if((uint16_t)(millis() - tStart) >= Ftimeout){
				//Timeout
				serial.println("  !!!TO, readBytes, i2cBusyFlag");
				return -1;
			}
		}
		while(I2C_GetFlagStatus(I2C_Module,I2C_FLAG_BUSY)== SET){
			if((uint16_t)(millis() - tStart) >= Ftimeout){
				//Timeout
				serial.println("  !!!TO, readBytes, Bus busy");
				return -1;
			}
		}


		for (int8_t subcount = 0; subcount < min(length, BUFFER_LENGTH)  && (Ftimeout == 0 || (uint16_t)(millis() - tStart)); subcount++) {
			//serial.printlnVal("count", count);

			data[count++] = rxBuffer[subcount];
			//serial.println("receive");
			#ifdef I2CDEV_SERIAL_DEBUG
				serial.print(data[count -1], 16);
				if (count + 1 < length) serial.print(" ");
			#endif
		}
	}
    // check for timeout
    if (Ftimeout > 0 && (uint16_t)(millis() - tStart) >= Ftimeout && count < length) count = -1; // timeout

    #ifdef I2CDEV_SERIAL_DEBUG
        serial.print(". Done (");
        serial.print(count, 10);
        serial.println(" read).");
    #endif

        //serial.println("will return");
    return count;
}

/** Read multiple words from a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr First register regAddr to read from
 * @param length Number of words to read
 * @param data Buffer to store read data in
 * @param timeout Optional read timeout in milliseconds (0 to disable, leave off to use default class value in I2Cdev::readTimeout)
 * @return Number of words read (-1 indicates failure)
 */
/*
int8_t I2Cdev::readWords(uint8_t devAddr, uint8_t regAddr, uint8_t length, uint16_t *data, uint16_t timeout) {
    #ifdef I2CDEV_SERIAL_DEBUG
        Serial.print("I2C (0x");
        Serial.print(devAddr, HEX);
        Serial.print(") reading ");
        Serial.print(length, DEC);
        Serial.print(" words from 0x");
        Serial.print(regAddr, HEX);
        Serial.print("...");
    #endif

    int8_t count = 0;
    uint32_t t1 = millis();

    #if (I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE)

        #if (ARDUINO < 100)
            // Arduino v00xx (before v1.0), Wire library

            // I2C/TWI subsystem uses internal buffer that breaks with large data requests
            // so if user requests more than BUFFER_LENGTH bytes, we have to do it in
            // smaller chunks instead of all at once
            for (uint8_t k = 0; k < length * 2; k += min(length * 2, BUFFER_LENGTH)) {
                Wire.beginTransmission(devAddr);
                Wire.send(regAddr);
                Wire.endTransmission();
                Wire.beginTransmission(devAddr);
                Wire.requestFrom(devAddr, (uint8_t)(length * 2)); // length=words, this wants bytes
    
                bool msb = true; // starts with MSB, then LSB
                for (; Wire.available() && count < length && (timeout == 0 || millis() - t1 < timeout);) {
                    if (msb) {
                        // first byte is bits 15-8 (MSb=15)
                        data[count] = Wire.receive() << 8;
                    } else {
                        // second byte is bits 7-0 (LSb=0)
                        data[count] |= Wire.receive();
                        #ifdef I2CDEV_SERIAL_DEBUG
                            Serial.print(data[count], HEX);
                            if (count + 1 < length) Serial.print(" ");
                        #endif
                        count++;
                    }
                    msb = !msb;
                }

                Wire.endTransmission();
            }
        #elif (ARDUINO == 100)
            // Arduino v1.0.0, Wire library
            // Adds standardized write() and read() stream methods instead of send() and receive()
    
            // I2C/TWI subsystem uses internal buffer that breaks with large data requests
            // so if user requests more than BUFFER_LENGTH bytes, we have to do it in
            // smaller chunks instead of all at once
            for (uint8_t k = 0; k < length * 2; k += min(length * 2, BUFFER_LENGTH)) {
                Wire.beginTransmission(devAddr);
                Wire.write(regAddr);
                Wire.endTransmission();
                Wire.beginTransmission(devAddr);
                Wire.requestFrom(devAddr, (uint8_t)(length * 2)); // length=words, this wants bytes
    
                bool msb = true; // starts with MSB, then LSB
                for (; Wire.available() && count < length && (timeout == 0 || millis() - t1 < timeout);) {
                    if (msb) {
                        // first byte is bits 15-8 (MSb=15)
                        data[count] = Wire.read() << 8;
                    } else {
                        // second byte is bits 7-0 (LSb=0)
                        data[count] |= Wire.read();
                        #ifdef I2CDEV_SERIAL_DEBUG
                            Serial.print(data[count], HEX);
                            if (count + 1 < length) Serial.print(" ");
                        #endif
                        count++;
                    }
                    msb = !msb;
                }
        
                Wire.endTransmission();
            }
        #elif (ARDUINO > 100)
            // Arduino v1.0.1+, Wire library
            // Adds official support for repeated start condition, yay!

            // I2C/TWI subsystem uses internal buffer that breaks with large data requests
            // so if user requests more than BUFFER_LENGTH bytes, we have to do it in
            // smaller chunks instead of all at once
            for (uint8_t k = 0; k < length * 2; k += min(length * 2, BUFFER_LENGTH)) {
                Wire.beginTransmission(devAddr);
                Wire.write(regAddr);
                Wire.endTransmission();
                Wire.beginTransmission(devAddr);
                Wire.requestFrom(devAddr, (uint8_t)(length * 2)); // length=words, this wants bytes
        
                bool msb = true; // starts with MSB, then LSB
                for (; Wire.available() && count < length && (timeout == 0 || millis() - t1 < timeout);) {
                    if (msb) {
                        // first byte is bits 15-8 (MSb=15)
                        data[count] = Wire.read() << 8;
                    } else {
                        // second byte is bits 7-0 (LSb=0)
                        data[count] |= Wire.read();
                        #ifdef I2CDEV_SERIAL_DEBUG
                            Serial.print(data[count], HEX);
                            if (count + 1 < length) Serial.print(" ");
                        #endif
                        count++;
                    }
                    msb = !msb;
                }
        
                Wire.endTransmission();
            }
        #endif

    #elif (I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE)

        // Fastwire library
        // no loop required for fastwire
        uint16_t intermediate[(uint8_t)length];
        uint8_t status = Fastwire::readBuf(devAddr << 1, regAddr, (uint8_t *)intermediate, (uint8_t)(length * 2));
        if (status == 0) {
            count = length; // success
            for (uint8_t i = 0; i < length; i++) {
                data[i] = (intermediate[2*i] << 8) | intermediate[2*i + 1];
            }
        } else {
            count = -1; // error
        }

    #endif

    if (timeout > 0 && millis() - t1 >= timeout && count < length) count = -1; // timeout

    #ifdef I2CDEV_SERIAL_DEBUG
        Serial.print(". Done (");
        Serial.print(count, DEC);
        Serial.println(" read).");
    #endif
    
    return count;
}
*/

/** write a single bit in an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to write to
 * @param bitNum Bit position to write (0-7)
 * @param value New bit value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeBit(uint8_t devAddr, uint8_t regAddr, uint8_t bitNum, uint8_t data) {
    uint8_t b;
    readByte(devAddr, regAddr, &b);
    b = (data != 0) ? (b | (1 << bitNum)) : (b & ~(1 << bitNum));
    return writeByte(devAddr, regAddr, b);
}

/** write a single bit in a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to write to
 * @param bitNum Bit position to write (0-15)
 * @param value New bit value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeBitW(uint8_t devAddr, uint8_t regAddr, uint8_t bitNum, uint16_t data) {
    uint16_t w;
    readWord(devAddr, regAddr, &w);
    w = (data != 0) ? (w | (1 << bitNum)) : (w & ~(1 << bitNum));
    return writeWord(devAddr, regAddr, w);
}

/** Write multiple bits in an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to write to
 * @param bitStart First bit position to write (0-7)
 * @param length Number of bits to write (not more than 8)
 * @param data Right-aligned value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeBits(uint8_t devAddr, uint8_t regAddr, uint8_t bitStart, uint8_t length, uint8_t data) {
    //      010 value to write
    // 76543210 bit numbers
    //    xxx   args: bitStart=4, length=3
    // 00011100 mask byte
    // 10101111 original value (sample)
    // 10100011 original & ~mask
    // 10101011 masked | value
    uint8_t b;
    if (readByte(devAddr, regAddr, &b) != 0) {
        uint8_t mask = ((1 << length) - 1) << (bitStart - length + 1);
        data <<= (bitStart - length + 1); // shift data into correct position
        data &= mask; // zero all non-important bits in data
        b &= ~(mask); // zero all important bits in existing byte
        b |= data; // combine data with existing byte
        return writeByte(devAddr, regAddr, b);
    } else {
        return false;
    }
}

/** Write multiple bits in a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register regAddr to write to
 * @param bitStart First bit position to write (0-15)
 * @param length Number of bits to write (not more than 16)
 * @param data Right-aligned value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeBitsW(uint8_t devAddr, uint8_t regAddr, uint8_t bitStart, uint8_t length, uint16_t data) {
    //              010 value to write
    // fedcba9876543210 bit numbers
    //    xxx           args: bitStart=12, length=3
    // 0001110000000000 mask word
    // 1010111110010110 original value (sample)
    // 1010001110010110 original & ~mask
    // 1010101110010110 masked | value
    uint16_t w;
    if (readWord(devAddr, regAddr, &w) != 0) {
        uint16_t mask = ((1 << length) - 1) << (bitStart - length + 1);
        data <<= (bitStart - length + 1); // shift data into correct position
        data &= mask; // zero all non-important bits in data
        w &= ~(mask); // zero all important bits in existing word
        w |= data; // combine data with existing word
        return writeWord(devAddr, regAddr, w);
    } else {
        return false;
    }
}

/** Write single byte to an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register address to write to
 * @param data New byte value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeByte(uint8_t devAddr, uint8_t regAddr, uint8_t data) {
    //return writeBytes(devAddr, regAddr, 1, &data);
	 bool rw =  writeBytes(devAddr, regAddr, 1, &data);

  //  while(i2cBusyFlag);
return rw;

}

/** Write single word to a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr Register address to write to
 * @param data New word value to write
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeWord(uint8_t devAddr, uint8_t regAddr, uint16_t data) {
    return writeWords(devAddr, regAddr, 1, &data);
}

/** Write multiple bytes to an 8-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr First register address to write to
 * @param length Number of bytes to write
 * @param data Buffer to copy new data from
 * @return Status of operation (true = success)
 */
bool I2Cdev::writeBytes(uint8_t devAddr, uint8_t _regAddr, uint8_t length, uint8_t* data) {
    #ifdef I2CDEV_SERIAL_DEBUG
        serial.print("I2Cdev (0x");
        serial.printVal(devAddr, 16);
        serial.print(") writing ");
        serial.printVal(length, 10);
        serial.print(" bytes to 0x");
        serial.printVal(_regAddr, 16);
        serial.print("...");
    #endif
    bool status = false;

    uint16_t tStart = millis();

	while(i2cBusyFlag){
		if((uint16_t)(millis() - tStart) >= I2CDEV_DEFAULT_READ_TIMEOUT){
			//Timeout
			#ifdef I2CDEV_SERIAL_DEBUG
			serial.println("TO I2C writeBytes, i2cBusyFlag");
			#endif
			return true;
		}
	}
    while(I2C_GetFlagStatus(I2C_Module,I2C_FLAG_BUSY)== SET){
    	if((uint16_t)(millis() - tStart) >= I2CDEV_DEFAULT_READ_TIMEOUT){
			#ifdef I2CDEV_SERIAL_DEBUG
			serial.println("TO I2C writeBytes, Bus busy");
			#endif
    		return true;
    	}
    }

	deviceAddress = devAddr;
	i2cDirectionWrite = 1;
	i2cBusyFlag = 1;

	regAddr = _regAddr;

    for (uint8_t i = 0; i < length; i++) {
        #ifdef I2CDEV_SERIAL_DEBUG
            serial.printVal(data[i], 16);
            if (i + 1 < length) serial.print(" ");
        #endif
		txBuffer[i] =  data[i];
    }
    txCount = length;
    txBufferIndex = 0;
    I2C_GenerateSTART(I2C_Module, ENABLE);

    #ifdef I2CDEV_SERIAL_DEBUG
        serial.println(". Done.");
    #endif
    return status == 0;
}

/** Write multiple words to a 16-bit device register.
 * @param devAddr I2C slave device address
 * @param regAddr First register address to write to
 * @param length Number of words to write
 * @param data Buffer to copy new data from
 * @return Status of operation (true = success)
 */

bool I2Cdev::writeWords(uint8_t devAddr, uint8_t myregAddr, uint8_t length, uint16_t* data) {
    #ifdef I2CDEV_SERIAL_DEBUG
        serial.print("I2C (0x");
        serial.print(devAddr, 16);
        serial.print(") writing ");
        serial.print(length, 10);
        serial.print(" words to 0x");
        serial.print(regAddr, 16);
        serial.print("...");
    #endif

     bool status = false;


     uint16_t tStart = millis();
	while(i2cBusyFlag){
	//serial.println("waiting1");
		if((uint16_t)(millis() - tStart) >= I2CDEV_DEFAULT_READ_TIMEOUT){
			//Timeout
			#ifdef I2CDEV_SERIAL_DEBUG
			serial.println("TO I2C writeWords, i2cBusyFlag");
			#endif
			return true;
		}
	}
    while(I2C_GetFlagStatus(I2C_Module,I2C_FLAG_BUSY)== SET){
    	//serial.println("waiting2");
    	if((uint16_t)(millis() - tStart) >= I2CDEV_DEFAULT_READ_TIMEOUT){
			#ifdef I2CDEV_SERIAL_DEBUG
			serial.println("TO I2C writeWords, Bus busy");
			#endif
    		return true;
    	}
    }
	deviceAddress = devAddr;
	i2cDirectionWrite = 1;
	i2cBusyFlag = 1;

	regAddr = myregAddr;

for (uint8_t i = 0; i < length *2 ; i++) {
    #ifdef I2CDEV_SERIAL_DEBUG
        serial.printVal(data[i], 16);
        if (i + 1 < length) serial.print(" ");
    #endif

      //  Wire.send((uint8_t) data[i]);
        txBuffer[i*2] =  (uint8_t)(data[i] >> 8);
        txBuffer[(i*2) + 1] =  (uint8_t)data[i];
        i++;
       // Wire.write((uint8_t)(data[i] >> 8));    // send MSB
       //             Wire.write((uint8_t)data[i++]);         // send LSB

}
txCount = (length* 2);
txBufferIndex = 0;
I2C_GenerateSTART(I2C_Module, ENABLE);
   // Wire.endTransmission();



#ifdef I2CDEV_SERIAL_DEBUG
    serial.println(". Done.");
#endif
return status == 0;
}



